/*
	mergeClips plugin for Avisynth -- 

    merge clip2 over clip1 in position x,y and transparency t

    No warranties about this plugin, implied or not; use at your own risk.
	Copyright (C) 2003 Sergio Rabellino, All Rights Reserved

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	The author can be contacted at:
	Sergio Rabellino
	rabellino@nospam-please.di.unito.it.
*/

// Include for avisynth API
#include "stdafx.h"
#include "avisynth.h"


class mergeClips : public GenericVideoFilter {
	PClip child2;
	int xpointer;
    int ypointer;
    int transparency;
	VideoInfo vi2;
	int row_size1; 
    int height1; 
	int row_size2; 
    int height2; 
	int alpha1;
	int alpha2;
    int alpha;

void init (IScriptEnvironment* env);

public:

    mergeClips (PClip _child1,
		        PClip _child2,
				int _xpointer,
				int _ypointer,
				int _transparency,
				IScriptEnvironment* env) :
	GenericVideoFilter(_child1),
			           child2(_child2),
			           xpointer(_xpointer),
			           ypointer(_ypointer),
			           transparency(_transparency)
	{ init(env);} 
    
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env); 

};

void mergeClips::init (IScriptEnvironment* env) {
	vi2=child2->GetVideoInfo();

	if (!vi.IsRGB32())
			env->ThrowError("mergeClips: clip1 RGB32 color space required");

	if (!vi2.IsRGB32())
			env->ThrowError("mergeClips: clip2 RGB32 color space required");

	if (transparency<-1 || transparency > 100)
			env->ThrowError("mergeClips: transparency must be between 0 to 100");

    row_size1 = vi.width; 
    height1 = vi.height; 

    row_size2 = vi2.width;
    height2 = vi2.height; 

    alpha1=100-transparency;
    alpha2=transparency;
    alpha=alpha1+alpha2;

	if ( ypointer + height2 > height1 )
			env->ThrowError("mergeClips: clip2+yoffset is more then clip1 height");

	if ( xpointer + row_size2 > row_size1 )
			env->ThrowError("mergeClips: clip2+xoffset is more then clip1 width");


}

PVideoFrame mergeClips::GetFrame(int n, IScriptEnvironment* env) {


    PVideoFrame frame1 = child->GetFrame(n, env); 
    PVideoFrame frame2 = child2->GetFrame(n, env); 

    env->MakeWritable(&frame1); 

    unsigned char* p1 = frame1->GetWritePtr(); 
    const int pitch1 = frame1->GetPitch(); 
 
    unsigned char* p2 = frame2->GetWritePtr(); 
    const int pitch2 = frame2->GetPitch(); 

    for (int g = 0; g < ypointer; g++)
      p1 += pitch1;

	if ( n < vi2.num_frames )
    for (int y = 0; y < height2; y++) {

        for (int x = 0; x < row_size2; x++)
		{
	      if(transparency > 0)
		  {
			// Blending clip2 over clip1 by transparency
			p1[(x+xpointer)*4+0] = (p2[x*4+0]*alpha2+p1[(x+xpointer)*4+0]*alpha1)/alpha; //B
            p1[(x+xpointer)*4+1] = (p2[x*4+1]*alpha2+p1[(x+xpointer)*4+1]*alpha1)/alpha; //G
            p1[(x+xpointer)*4+2] = (p2[x*4+2]*alpha2+p1[(x+xpointer)*4+2]*alpha1)/alpha; //R
            //p[x*4+3] = 255; //Alpha
		  }
		  if ( transparency == 0 )
  		  {
			// Keying clip2 over clip1 if gray value is high
			  p1[(x+xpointer)*4+0] *= (unsigned char)(p2[x*4+0]/255.0); //B
              p1[(x+xpointer)*4+1] *= (unsigned char)(p2[x*4+1]/255.0); //G
              p1[(x+xpointer)*4+2] *= (unsigned char)(p2[x*4+2]/255.0); //R
            //p[x*4+3] = 255; //Alpha
		  }

		  if ( transparency == -1 )
  		  {
			// Keying clip2 over clip1 if gray value is high
			  if ( p2[(x)*4+0]+
				   p2[(x)*4+1]+
				   p2[(x)*4+2] > 384  )
			  {
			    p1[(x+xpointer)*4+0] = p2[x*4+0]; //B
                p1[(x+xpointer)*4+1] = p2[x*4+1]; //G
                p1[(x+xpointer)*4+2] = p2[x*4+2]; //R
                //p[x*4+3] = 255; //Alpha
			  }
			 
		  }

		}
    p1 += pitch1;
	p2 += pitch2;
    } 
    return frame1;
}

AVSValue __cdecl Create_mergeClips(AVSValue args, void* user_data, IScriptEnvironment* env) {

    return new mergeClips(args[0].AsClip(),
			  args[1].AsClip(),
		          args[2].AsInt(0),
		          args[3].AsInt(0),
		          args[4].AsInt(0),
			  env ); 

}

extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) {

    env->AddFunction("MergeClips", "c[clip2]c[xpos]i[ypos]i[alpha]i", Create_mergeClips, 0); 
        // The AddFunction has the following paramters:
    // AddFunction(Filtername , Arguments, Function to call,0);
    
    // Arguments is a string that defines the types and optional names of the arguments for you filter.
    // c - Video Clip
    // i - Integer number
    // f - Float number
    // s - String
    // b - boolean

	 // The word inside the [ ] lets you used named parameters in your script
    return "MergeClips plugin"; 

}
